/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2004  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "types.h"
#include "keyboard.h"
#include "pia.h"
#include "logging.h"

/* These contain masks to be applied when the corresponding row/column is
 * held low.  eg, if row 1 is outputting a 0 , keyboard_column[1] will
 * be applied on column reads */
Uint32 keyboard_column[8];
Uint32 keyboard_row[8];

Uint32 keyboard_last_column_mask;
Uint32 keyboard_last_row_mask;

void keyboard_init(void) {
	/* Register PIA devices */
	//PIA_0B.wrfunc[0] = keyboard_column_update;
	//PIA_0A.wrfunc[0] = keyboard_row_update;
	keyboard_init_arch();
}

void keyboard_reset(void) {
	Uint32 i;
	for (i = 0; i < 8; i++) {
		keyboard_column[i] = ~0;
		keyboard_row[i] = ~0;
	}
	keyboard_last_column_mask = 0xff;
	keyboard_last_row_mask = 0xff;
}

void keyboard_column_update(void) {
	Uint32 mask = PIA_0B.port_output;
	Uint32 i, row = 0x7f;
	keyboard_last_column_mask = mask;
	for (i = 0; i < 8; i++) {
		if (!(mask & (1 << i))) {
			row &= keyboard_column[i];
		}
	}
	PIA_0A.port_input = (PIA_0A.port_input & 0x80) | row;
}

void keyboard_row_update(void) {
	Uint32 mask = PIA_0A.port_output;
	Uint32 i, col = 0xff;
	keyboard_last_row_mask = mask;
	for (i = 0; i < 7; i++) {
		if (!(mask & (1 << i))) {
			col &= keyboard_row[i];
		}
	}
	PIA_0B.port_input = col;
}
