/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2004  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "m6809.h"
#include "tape.h"
#include "fs.h"
#include "ui.h"
#include "types.h"
#include "logging.h"

static FS_FILE tapefd;
static Uint32 count = 0;
static Uint8 tape_byte = 0;
static Uint32 isopen = 0;
static char buffer[512];
static Uint32 bufferpos;

void tape_init(void) {
	isopen = 0;
}

void tape_reset(void) {
	if (isopen)
		fs_close(tapefd);
	isopen = 0;
}

/* Close any currently-open tape file, open a new one and read the first
 * bufferful of data */
int tape_attach(char *filename) {
	if (isopen)
		fs_close(tapefd);
	isopen = 0;
	if ((tapefd = fs_open(filename, FS_READ)) == -1)
		return -1;
	isopen = 1;
	fs_read(tapefd, buffer, sizeof(buffer));
	bufferpos = 0;
	return 0;
}

void tape_detach(void) {
	if (isopen)
		fs_close(tapefd);
}

void tape_read_bit(void) {
	if (!isopen && tape_attach(ui_get_filename("CAS")) == -1)
		return;
	if (!count) {
		tape_byte = buffer[bufferpos++];
		count = 8;
		if (bufferpos >= sizeof(buffer)) {
			fs_read(tapefd, buffer, sizeof(buffer));
			bufferpos = 0;
		}
	}
	reg_cc &= ~1;
	reg_cc |= (tape_byte & 1);
	tape_byte >>= 1;
	count--;
}
