/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2004  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "video.h"
#include "types.h"

static PIXEL backup[320*240];

GPDRAWSURFACE screen;

void video_init(void) {
	GpGraphicModeSet(VIDEO_BPP, NULL);
	GpLcdSurfaceGet(&screen, 0);
	GpSurfaceSet(&screen);
	GpLcdEnable();
	video_fillrect(0,0,320,194,MAPCOLOUR(0,0,0));
	video_fillrect(0,194,320,46,MAPCOLOUR(255,255,255));
}

void video_fillrect(Uint32 x, Uint32 y, Uint32 w, Uint32 h, PIXEL colour) {
	PIXEL *p = (PIXEL *)screen.ptbuffer + x * 240 + (239-y);
	Uint32 skip = 240 + h;
	Uint32 j;
	for (; w; w--) {
		for (j = h; j; j--) {
			*(p--) = colour;
		}
		p += skip;
	}
}

void video_blit(Uint32 x, Uint32 y, PIXEL *src, Uint32 w, Uint32 h) {
	PIXEL *p = VIDEO_SCREENBASE + (x * 240) + (240-y) - h;
	Uint32 j;
	for (; w; w--) {
		for (j = h; j; j--) {
			*(p++) = *(src++);
		}
		p += (240 - h);
	}
}

void video_backup(void) {
	memcpy(backup, VIDEO_SCREENBASE, 320*240*sizeof(PIXEL));
}

void video_restore(void) {
	memcpy(VIDEO_SCREENBASE, backup, 320*240*sizeof(PIXEL));
}
